import { Body, Controller, Get, Param, Post, Query, Req } from '@nestjs/common';
import { CreateStructureDto } from './dto/create-structure.dto';
import { QueryStructure } from './dto/query-structure.dto';
import { Structure } from './schemas/structure.schema';
import { StructuresService } from './structures.service';

@Controller('structures')
export class StructuresController {
  constructor(private readonly structureService: StructuresService) {}

  @Post()
  public async create(@Body() createStructureDto: CreateStructureDto) {
    await this.structureService.create(createStructureDto);
  }

  @Post('search')
  public async search(@Query() query: QueryStructure, @Body() body): Promise<Structure[]> {
    return this.structureService.search(query.query, body ? body.filters : null);
  }

  @Get()
  public async findAll(): Promise<Structure[]> {
    return this.structureService.findAll();
  }

  @Get('count')
  public async countCategories(): Promise<Array<{ id: string; count: number }>> {
    const data = await Promise.all([
      this.structureService.countByStructureKey('accesAuxDroits'),
      this.structureService.countByStructureKey('aideALaParentalite'),
      this.structureService.countByStructureKey('cultureEtSecuriteNumerique'),
      this.structureService.countByStructureKey('insertionSocialeEtProfessionnelle'),
      this.structureService.countByStructureKey('accompagnementDesDemarches'),
      this.structureService.countByStructureKey('labelsEtQualifications'),
      this.structureService.countByStructureKey('publicsAcceptes'),
      this.structureService.countByStructureKey('modalitesDacces'),
      this.structureService.countByStructureKey('lesCompetencesDeBase'),
      this.structureService.countByEquipmentsKey('wifiEnAccesLibre', 'Wifi en accès libre'),
      this.structureService.countByEquipmentsKey('ordinateurs', 'Ordinateurs'),
      this.structureService.countByEquipmentsKey('tablettes', 'Tablettes'),
      this.structureService.countByEquipmentsKey('bornesNumeriques', 'Bornes numériques'),
      this.structureService.countByEquipmentsKey('imprimantes', 'Imprimantes'),
    ]);
    // Return a concat of all arrays
    return data.reduce((a, b) => [...a, ...b]);
  }
}
